@@ -3,6 +3,7 @@
  *  Wireless daemon for Linux
  *
  *  Copyright (C) 2017-2019  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2023, Locus Robotics
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -34,8 +35,12 @@ struct proxy_interface;
 #define IWD_WSC_INTERFACE                "net.connman.iwd.SimpleConfiguration"
 #define IWD_STATION_INTERFACE            "net.connman.iwd.Station"
 #define IWD_STATION_DIAGNOSTIC_INTERFACE "net.connman.iwd.StationDiagnostic"
+#define IWD_STATION_DEBUG_INTERFACE      "net.connman.iwd.StationDebug"
 #define IWD_AP_DIAGNOSTIC_INTERFACE      "net.connman.iwd.AccessPointDiagnostic"
 #define IWD_DAEMON_INTERFACE             "net.connman.iwd.Daemon"
+#define IWD_DPP_INTERFACE                "net.connman.iwd.DeviceProvisioning"
+#define IWD_DPP_PKEX_INTERFACE \
+				"net.connman.iwd.SharedCodeDeviceProvisioning"
 
 typedef bool (*proxy_property_match_func_t) (const void *a, const void *b);
 
@@ -88,7 +93,8 @@ bool proxy_interface_method_call(const struct proxy_interface *proxy,
 
 void proxy_properties_display(const struct proxy_interface *proxy,
 				const char *caption, const char *margin,
-				int name_column_width, int value_column_width);
+				unsigned int name_column_width,
+				unsigned int value_column_width);
 
 char *proxy_property_str_completion(const struct proxy_interface_type *type,
 					proxy_property_match_func_t function,
@@ -118,10 +124,14 @@ struct interface_type_desc {
 } __attribute__((aligned(8)));
 
 #define INTERFACE_TYPE(interface, init, exit)				\
+	_Pragma("GCC diagnostic push")					\
+	_Pragma("GCC diagnostic ignored \"-Wattributes\"")		\
 	static struct interface_type_desc __interface_type_ ## interface\
-		__attribute__((used, section("__interface"), aligned(8))) = {\
+		__attribute__((used, retain, section("__interface"),	\
+					aligned(8))) = {		\
 			#interface, init, exit				\
 		};							\
+	_Pragma("GCC diagnostic pop")
 
 bool dbus_proxy_init(void);
 bool dbus_proxy_exit(void);
