@@ -1,23 +1,8 @@
 /*
+ * Embedded Linux library
+ * Copyright (C) 2011-2014  Intel Corporation
  *
- *  Embedded Linux library
- *
- *  Copyright (C) 2011-2014  Intel Corporation. All rights reserved.
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 2.1 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
- *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #ifdef HAVE_CONFIG_H
@@ -1655,11 +1640,11 @@ bool _dbus_object_tree_remove_interface(struct _dbus_object_tree *tree,
 		if (interfaces_added_rec && l_queue_remove(
 						interfaces_added_rec->instances,
 						instance)) {
-			if (l_queue_isempty(interfaces_added_rec->instances))
+			if (l_queue_isempty(interfaces_added_rec->instances)) {
 				l_queue_remove(manager->announce_added,
 						interfaces_added_rec);
-
-			interface_add_record_free(interfaces_added_rec);
+				interface_add_record_free(interfaces_added_rec);
+			}
 
 			continue;
 		}
