@@ -156,6 +156,7 @@ static bool anqp_response_frame_event(const struct mmpdu_header *hdr,
 }
 
 static const struct frame_xchg_prefix anqp_frame_prefix = {
+	.frame_type = 0x00d0,
 	.data = (uint8_t []) {
 		0x04, 0x0b,
 	},
@@ -256,7 +257,7 @@ uint32_t anqp_request(uint64_t wdev_id, const uint8_t *addr,
 				&anqp_frame_prefix, anqp_response_frame_event,
 				NULL);
 
-	return true;
+	return request->id;
 }
 
 void anqp_cancel(uint32_t id)
