@@ -121,8 +121,8 @@ enum p2p_public_action_frame_type {
 /* Table 75 */
 enum p2p_action_frame_type {
 	P2P_ACTION_NOTICE_OF_ABSENCE		= 0,
-	P2P_ACTION_PRESENCE_REQ 		= 1,
-	P2P_ACTION_PRESENCE_RESP 		= 2,
+	P2P_ACTION_PRESENCE_REQ			= 1,
+	P2P_ACTION_PRESENCE_RESP		= 2,
 	P2P_ACTION_GO_DISCOVERABILITY_REQ	= 3,
 };
 
@@ -152,6 +152,73 @@ static inline const uint8_t *p2p_attr_iter_get_data(struct p2p_attr_iter *iter)
 	return iter->pos + 3;
 }
 
+/* Wi-Fi Display Technical Specification v2.1.0 Table 27 */
+enum wfd_subelem_type {
+	WFD_SUBELEM_WFD_DEVICE_INFORMATION	= 0,
+	WFD_SUBELEM_ASSOCIATED_BSSID		= 1,
+	WFD_SUBELEM_COUPLED_SINK_INFORMATION	= 6,
+	WFD_SUBELEM_EXTENDED_CAPABILITY		= 7,
+	WFD_SUBELEM_LOCAL_IP_ADDRESS		= 8,
+	WFD_SUBELEM_SESION_INFORMATION		= 9,
+	WFD_SUBELEM_ALTERNATIVE_MAC_ADDRESS	= 10,
+	WFD_SUBELEM_R2_DEVICE_INFORMATION	= 11,
+};
+
+enum wfd_dev_info_bits {
+	WFD_DEV_INFO_DEVICE_TYPE		= 0x0003,
+	WFD_DEV_INFO_COUPLED_SINK_AT_SOURCE_OK	= 0x0004,
+	WFD_DEV_INFO_COUPLED_SINK_AT_SINK_OK	= 0x0008,
+	WFD_DEV_INFO_SESSION_AVAILABILITY	= 0x0030,
+	WFD_DEV_INFO_SERVICE_DISCOVERY_SUPPORT	= 0x0040,
+	WFD_DEV_INFO_PREFER_TDLS_CONNECTIVITY	= 0x0080,
+	WFD_DEV_INFO_CONTENT_PROTECTION_SUPPORT	= 0x0100,
+	WFD_DEV_INFO_8021AS_TIME_SYNC_SUPPORT	= 0x0200,
+	WFD_DEV_INFO_NO_AUDIO_AT_PRIMARY_SINK	= 0x0400,
+	WFD_DEV_INFO_AUDIO_ONLY_AT_SOURCE	= 0x0800,
+	WFD_DEV_INFO_TDLS_PERSISTENT_GROUP	= 0x1000,
+	WFD_DEV_INFO_REINVOKE_TDLS_GROUP	= 0x2000,
+};
+
+enum wfd_dev_info_type {
+	WFD_DEV_INFO_TYPE_SOURCE		= 0x0000,
+	WFD_DEV_INFO_TYPE_PRIMARY_SINK		= 0x0001,
+	WFD_DEV_INFO_TYPE_SECONDARY_SINK	= 0x0002,
+	WFD_DEV_INFO_TYPE_DUAL_ROLE		= 0x0003,
+};
+
+enum wfd_dev_info_session_availability {
+	WFD_DEV_INFO_SESSION_NOT_AVAILABLE	= 0x0000,
+	WFD_DEV_INFO_SESSION_AVAILABLE		= 0x0010,
+};
+
+struct wfd_subelem_iter {
+	const uint8_t *pos;
+	const uint8_t *end;
+	enum wfd_subelem_type type;
+	size_t len;
+};
+
+void wfd_subelem_iter_init(struct wfd_subelem_iter *iter, const uint8_t *pdu,
+				size_t len);
+bool wfd_subelem_iter_next(struct wfd_subelem_iter *iter);
+
+static inline enum wfd_subelem_type wfd_subelem_iter_get_type(
+						struct wfd_subelem_iter *iter)
+{
+	return iter->type;
+}
+
+static inline size_t wfd_subelem_iter_get_length(struct wfd_subelem_iter *iter)
+{
+	return iter->len;
+}
+
+static inline const uint8_t *wfd_subelem_iter_get_data(
+						struct wfd_subelem_iter *iter)
+{
+	return iter->pos + 3;
+}
+
 struct p2p_capability_attr {
 	uint8_t device_caps;
 	uint8_t group_caps;
@@ -310,6 +377,8 @@ struct p2p_go_negotiation_req {
 	struct p2p_device_info_attr device_info;
 	struct p2p_channel_attr operating_channel;
 	enum wsc_device_password_id device_password_id;
+	uint8_t *wfd;
+	ssize_t wfd_size;
 };
 
 struct p2p_go_negotiation_resp {
@@ -325,6 +394,8 @@ struct p2p_go_negotiation_resp {
 	struct p2p_device_info_attr device_info;
 	struct p2p_group_id_attr group_id;
 	enum wsc_device_password_id device_password_id;
+	uint8_t *wfd;
+	ssize_t wfd_size;
 };
 
 struct p2p_go_negotiation_confirmation {
@@ -334,6 +405,8 @@ struct p2p_go_negotiation_confirmation {
 	struct p2p_channel_attr operating_channel;
 	struct p2p_channel_list_attr channel_list;
 	struct p2p_group_id_attr group_id;
+	uint8_t *wfd;
+	ssize_t wfd_size;
 };
 
 struct p2p_invitation_req {
@@ -346,6 +419,8 @@ struct p2p_invitation_req {
 	struct p2p_group_id_attr group_id;
 	struct p2p_device_info_attr device_info;
 	enum wsc_device_password_id device_password_id;
+	uint8_t *wfd;
+	ssize_t wfd_size;
 };
 
 struct p2p_invitation_resp {
@@ -355,6 +430,8 @@ struct p2p_invitation_resp {
 	struct p2p_channel_attr operating_channel;
 	uint8_t group_bssid[6];
 	struct p2p_channel_list_attr channel_list;
+	uint8_t *wfd;
+	ssize_t wfd_size;
 };
 
 struct p2p_device_discoverability_req {
@@ -386,6 +463,8 @@ struct p2p_provision_discovery_req {
 	enum p2p_asp_coordination_transport_protocol transport_protocol;
 	struct p2p_group_id_attr persistent_group_info;
 	uint16_t wsc_config_method;
+	uint8_t *wfd;
+	ssize_t wfd_size;
 };
 
 struct p2p_provision_discovery_resp {
@@ -405,6 +484,8 @@ struct p2p_provision_discovery_resp {
 	struct p2p_group_id_attr persistent_group_info;
 	struct p2p_session_info_data_attr session_info;
 	uint16_t wsc_config_method;
+	uint8_t *wfd;
+	ssize_t wfd_size;
 };
 
 struct p2p_notice_of_absence {
@@ -522,3 +603,5 @@ uint8_t *p2p_build_presence_req(const struct p2p_presence_req *data,
 uint8_t *p2p_build_presence_resp(const struct p2p_presence_resp *data,
 					size_t *out_len);
 uint8_t *p2p_build_go_disc_req(size_t *out_len);
+
+void p2p_get_random_string(char *buf, size_t len);
